"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "useOnCellActiveAnimation", {
  enumerable: true,
  get: function () {
    return _useOnCellActiveAnimation.useOnCellActiveAnimation;
  }
});
exports.OpacityDecorator = exports.ShadowDecorator = exports.ScaleDecorator = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));

var _useNode = require("../hooks/useNode");

var _useOnCellActiveAnimation = require("../hooks/useOnCellActiveAnimation");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// support older versions of Reanimated v1 by using the old interpolate function
// if interpolateNode not available.
const interpolateFn = _reactNativeReanimated.interpolateNode || _reactNativeReanimated.interpolate;

const ScaleDecorator = ({
  activeScale = 1.1,
  children
}) => {
  const {
    isActive,
    onActiveAnim
  } = (0, _useOnCellActiveAnimation.useOnCellActiveAnimation)({
    animationConfig: {
      mass: 0.1,
      restDisplacementThreshold: 0.0001
    }
  });
  const animScale = (0, _useNode.useNode)(interpolateFn(onActiveAnim, {
    inputRange: [0, 1],
    outputRange: [1, activeScale]
  }));
  const scale = isActive ? animScale : 1;
  return /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    style: {
      transform: [{
        scaleX: scale
      }, {
        scaleY: scale
      }]
    }
  }, children);
};

exports.ScaleDecorator = ScaleDecorator;

const ShadowDecorator = ({
  elevation = 10,
  color = "black",
  opacity = 0.25,
  radius = 5,
  children
}) => {
  const {
    isActive,
    onActiveAnim
  } = (0, _useOnCellActiveAnimation.useOnCellActiveAnimation)();
  const shadowOpacity = (0, _useNode.useNode)((0, _reactNativeReanimated.multiply)(onActiveAnim, opacity));
  const style = {
    elevation: isActive ? elevation : 0,
    shadowRadius: isActive ? radius : 0,
    shadowColor: isActive ? color : "transparent",
    shadowOpacity: isActive ? shadowOpacity : 0
  };
  return /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    style: style
  }, children);
};

exports.ShadowDecorator = ShadowDecorator;

const OpacityDecorator = ({
  activeOpacity = 0.25,
  children
}) => {
  const {
    isActive,
    onActiveAnim
  } = (0, _useOnCellActiveAnimation.useOnCellActiveAnimation)();
  const opacity = (0, _useNode.useNode)(interpolateFn(onActiveAnim, {
    inputRange: [0, 1],
    outputRange: [1, activeOpacity]
  }));
  const style = {
    opacity: isActive ? opacity : 1
  };
  return /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    style: style
  }, children);
};

exports.OpacityDecorator = OpacityDecorator;
//# sourceMappingURL=CellDecorators.js.map