"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _reactNative = require("react-native");

var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));

var _DraggableFlatListContext = require("../context/DraggableFlatListContext");

var _constants = require("../constants");

var _useCellTranslate = require("../hooks/useCellTranslate");

var _utils = require("../utils");

var _RefContext = require("../context/RefContext");

var _AnimatedValueContext = require("../context/AnimatedValueContext");

var _CellContext = _interopRequireDefault(require("../context/CellContext"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function CellRendererComponent(props) {
  const {
    item,
    index,
    onLayout,
    children
  } = props;
  const currentIndexAnim = (0, _reactNativeReanimated.useValue)(index);
  (0, _react.useLayoutEffect)(() => {
    currentIndexAnim.setValue(index);
  }, [index, currentIndexAnim]);
  const viewRef = (0, _react.useRef)(null);
  const {
    cellDataRef,
    propsRef,
    scrollOffsetRef,
    containerRef
  } = (0, _RefContext.useRefs)();
  const {
    horizontalAnim
  } = (0, _AnimatedValueContext.useAnimatedValues)();
  const {
    activeKey,
    keyExtractor,
    horizontal
  } = (0, _DraggableFlatListContext.useDraggableFlatListContext)();
  const key = keyExtractor(item, index);
  const offset = (0, _reactNativeReanimated.useValue)(-1);
  const size = (0, _reactNativeReanimated.useValue)(-1);
  const translate = (0, _useCellTranslate.useCellTranslate)({
    cellOffset: offset,
    cellSize: size,
    cellIndex: currentIndexAnim
  });
  (0, _react.useMemo)(() => {
    // prevent flicker on web
    if (_constants.isWeb) translate.setValue(0);
  }, [index]); //eslint-disable-line react-hooks/exhaustive-deps

  const isActive = activeKey === key;
  const style = (0, _react.useMemo)(() => ({
    transform: [{
      translateX: (0, _reactNativeReanimated.cond)(horizontalAnim, translate, 0)
    }, {
      translateY: (0, _reactNativeReanimated.cond)(horizontalAnim, 0, translate)
    }]
  }), [horizontalAnim, translate]);
  const updateCellMeasurements = (0, _react.useCallback)(() => {
    var _viewRef$current;

    const onSuccess = (x, y, w, h) => {
      if (_constants.isWeb && horizontal) x += scrollOffsetRef.current;
      const cellOffset = horizontal ? x : y;
      const cellSize = horizontal ? w : h;
      cellDataRef.current.set(key, {
        measurements: {
          size: cellSize,
          offset: cellOffset
        }
      });
      size.setValue(cellSize);
      offset.setValue(cellOffset);
    };

    const onFail = () => {
      var _propsRef$current;

      if ((_propsRef$current = propsRef.current) !== null && _propsRef$current !== void 0 && _propsRef$current.debug) {
        console.log(`## on measure fail, index: ${index}`);
      }
    }; // findNodeHandle is being deprecated. This is no longer necessary if using reanimated v2
    // remove once v1 is no longer supported


    const containerNode = containerRef.current;
    const viewNode = _constants.isReanimatedV2 ? viewRef.current : (_viewRef$current = viewRef.current) === null || _viewRef$current === void 0 ? void 0 : _viewRef$current.getNode(); //@ts-ignore

    const nodeHandle = _constants.isReanimatedV2 ? containerNode : (0, _reactNative.findNodeHandle)(containerNode);

    if (viewNode && nodeHandle) {
      //@ts-ignore
      viewNode.measureLayout(nodeHandle, onSuccess, onFail);
    }
  }, [cellDataRef, horizontal, index, key, offset, propsRef, size, scrollOffsetRef, containerRef]);
  (0, _react.useEffect)(() => {
    if (_constants.isWeb) {
      // onLayout isn't called on web when the cell index changes, so we manually re-measure
      updateCellMeasurements();
    }
  }, [index, updateCellMeasurements]);
  const onCellLayout = (0, _react.useCallback)(e => {
    updateCellMeasurements();
    onLayout(e);
  }, [updateCellMeasurements, onLayout]); // changing zIndex crashes android:
  // https://github.com/facebook/react-native/issues/28751

  return /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, _extends({}, props, {
    ref: viewRef,
    onLayout: onCellLayout,
    style: [_constants.isAndroid && {
      elevation: isActive ? 1 : 0
    }, {
      flexDirection: horizontal ? "row" : "column"
    }, (_constants.isWeb || _constants.isIOS) && {
      zIndex: isActive ? 999 : 0
    }],
    pointerEvents: activeKey ? "none" : "auto"
  }), /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    // Including both animated styles and non-animated styles causes react-native-web
    // to ignore updates in non-animated styles. Solution is to separate anima
    style: style
  }, /*#__PURE__*/_react.default.createElement(_CellContext.default, {
    isActive: isActive
  }, children)));
}

var _default = (0, _utils.typedMemo)(CellRendererComponent);

exports.default = _default;
//# sourceMappingURL=CellRendererComponent.js.map