"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = DraggableFlatList;

var _react = _interopRequireWildcard(require("react"));

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));

var _CellRendererComponent = _interopRequireDefault(require("./CellRendererComponent"));

var _constants = require("../constants");

var _PlaceholderItem = _interopRequireDefault(require("./PlaceholderItem"));

var _RowItem = _interopRequireDefault(require("./RowItem"));

var _ScrollOffsetListener = _interopRequireDefault(require("./ScrollOffsetListener"));

var _useAutoScroll = require("../hooks/useAutoScroll");

var _useNode = require("../hooks/useNode");

var _PropsContext = _interopRequireDefault(require("../context/PropsContext"));

var _AnimatedValueContext = _interopRequireWildcard(require("../context/AnimatedValueContext"));

var _RefContext = _interopRequireWildcard(require("../context/RefContext"));

var _DraggableFlatListContext = _interopRequireDefault(require("../context/DraggableFlatListContext"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const AnimatedFlatList = _reactNativeReanimated.default.createAnimatedComponent(_reactNativeGestureHandler.FlatList);

function DraggableFlatListInner(props) {
  const {
    cellDataRef,
    containerRef,
    flatlistRef,
    isTouchActiveRef,
    keyToIndexRef,
    panGestureHandlerRef,
    propsRef,
    scrollOffsetRef
  } = (0, _RefContext.useRefs)();
  const {
    activationDistance,
    activeCellOffset,
    activeCellSize,
    activeIndexAnim,
    containerSize,
    disabled,
    hasMoved,
    panGestureState,
    resetTouchedCell,
    scrollOffset,
    scrollViewSize,
    spacerIndexAnim,
    touchAbsolute,
    touchInit
  } = (0, _AnimatedValueContext.useAnimatedValues)();
  const {
    dragHitSlop = _constants.DEFAULT_PROPS.dragHitSlop,
    scrollEnabled = _constants.DEFAULT_PROPS.scrollEnabled,
    activationDistance: activationDistanceProp = _constants.DEFAULT_PROPS.activationDistance
  } = props;
  const [activeKey, setActiveKey] = (0, _react.useState)(null);
  const keyExtractor = (0, _react.useCallback)((item, index) => {
    if (propsRef.current.keyExtractor) return propsRef.current.keyExtractor(item, index);else throw new Error("You must provide a keyExtractor to DraggableFlatList");
  }, [propsRef]);
  (0, _react.useLayoutEffect)(() => {
    props.data.forEach((d, i) => {
      const key = keyExtractor(d, i);
      keyToIndexRef.current.set(key, i);
    });
  }, [props.data, keyExtractor, keyToIndexRef]);
  const drag = (0, _react.useCallback)(activeKey => {
    if (!isTouchActiveRef.current.js) return;
    const index = keyToIndexRef.current.get(activeKey);
    const cellData = cellDataRef.current.get(activeKey);

    if (cellData) {
      activeCellOffset.setValue(cellData.measurements.offset - scrollOffsetRef.current);
      activeCellSize.setValue(cellData.measurements.size);
    }

    const {
      onDragBegin
    } = propsRef.current;

    if (index !== undefined) {
      spacerIndexAnim.setValue(index);
      activeIndexAnim.setValue(index);
      setActiveKey(activeKey);
      onDragBegin === null || onDragBegin === void 0 ? void 0 : onDragBegin(index);
    }
  }, [isTouchActiveRef, keyToIndexRef, cellDataRef, propsRef, activeCellOffset, scrollOffsetRef, activeCellSize, spacerIndexAnim, activeIndexAnim]);
  const autoScrollNode = (0, _useAutoScroll.useAutoScroll)();

  const onContainerLayout = () => {
    var _containerRef$current;

    const containerNode = _constants.isReanimatedV2 ? containerRef.current : (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.getNode(); //@ts-ignore

    containerNode === null || containerNode === void 0 ? void 0 : containerNode.measure((_x, _y, w, h) => {
      containerSize.setValue(props.horizontal ? w : h);
    });
  };

  const onListContentSizeChange = (w, h) => {
    var _props$onContentSizeC;

    scrollViewSize.setValue(props.horizontal ? w : h);
    (_props$onContentSizeC = props.onContentSizeChange) === null || _props$onContentSizeC === void 0 ? void 0 : _props$onContentSizeC.call(props, w, h);
  };

  const onContainerTouchStart = () => {
    isTouchActiveRef.current.js = true;
    isTouchActiveRef.current.native.setValue(1);
    return false;
  };

  const onContainerTouchEnd = () => {
    isTouchActiveRef.current.js = false;
    isTouchActiveRef.current.native.setValue(0);
  };

  let dynamicProps = {};

  if (activationDistanceProp) {
    const activeOffset = [-activationDistanceProp, activationDistanceProp];
    dynamicProps = props.horizontal ? {
      activeOffsetX: activeOffset
    } : {
      activeOffsetY: activeOffset
    };
  }

  const extraData = (0, _react.useMemo)(() => ({
    activeKey,
    extraData: props.extraData
  }), [activeKey, props.extraData]);
  const renderItem = (0, _react.useCallback)(({
    item,
    index
  }) => {
    return /*#__PURE__*/_react.default.createElement(_RowItem.default, {
      item: item,
      itemKey: keyExtractor(item, index),
      renderItem: props.renderItem,
      drag: drag,
      extraData: props.extraData
    });
  }, [props.renderItem, props.extraData, drag, keyExtractor]);
  const resetHoverState = (0, _react.useCallback)(() => {
    activeIndexAnim.setValue(-1);
    spacerIndexAnim.setValue(-1);
    touchAbsolute.setValue(0);
    disabled.setValue(0);
    requestAnimationFrame(() => {
      setActiveKey(null);
    });
  }, [activeIndexAnim, spacerIndexAnim, touchAbsolute, disabled]);

  const onRelease = ([index]) => {
    var _props$onRelease;

    // This shouldn't be necessary but seems to fix a bug where sometimes
    // native values wouldn't update
    isTouchActiveRef.current.native.setValue(0);
    (_props$onRelease = props.onRelease) === null || _props$onRelease === void 0 ? void 0 : _props$onRelease.call(props, index);
  };

  const onDragEnd = (0, _react.useCallback)(([from, to]) => {
    const {
      onDragEnd,
      data
    } = propsRef.current;

    if (onDragEnd) {
      const newData = [...data];

      if (from !== to) {
        newData.splice(from, 1);
        newData.splice(to, 0, data[from]);
      }

      onDragEnd({
        from,
        to,
        data: newData
      });
    }

    resetHoverState();
  }, [resetHoverState, propsRef]);
  const onGestureRelease = (0, _useNode.useNode)((0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.greaterThan)(activeIndexAnim, -1), [(0, _reactNativeReanimated.set)(disabled, 1), (0, _reactNativeReanimated.set)(isTouchActiveRef.current.native, 0), (0, _reactNativeReanimated.call)([activeIndexAnim], onRelease), (0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.not)(hasMoved), (0, _reactNativeReanimated.call)([activeIndexAnim], resetHoverState))], [(0, _reactNativeReanimated.call)([activeIndexAnim], resetHoverState), resetTouchedCell]));
  const onPanStateChange = (0, _react.useMemo)(() => (0, _reactNativeReanimated.event)([{
    nativeEvent: ({
      state,
      x,
      y
    }) => (0, _reactNativeReanimated.block)([(0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.and)((0, _reactNativeReanimated.neq)(state, panGestureState), (0, _reactNativeReanimated.not)(disabled)), [(0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.or)((0, _reactNativeReanimated.eq)(state, _reactNativeGestureHandler.State.BEGAN), // Called on press in on Android, NOT on ios!
    // GestureState.BEGAN may be skipped on fast swipes
    (0, _reactNativeReanimated.and)((0, _reactNativeReanimated.eq)(state, _reactNativeGestureHandler.State.ACTIVE), (0, _reactNativeReanimated.neq)(panGestureState, _reactNativeGestureHandler.State.BEGAN))), [(0, _reactNativeReanimated.set)(touchAbsolute, props.horizontal ? x : y), (0, _reactNativeReanimated.set)(touchInit, touchAbsolute)]), (0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.eq)(state, _reactNativeGestureHandler.State.ACTIVE), [(0, _reactNativeReanimated.set)(activationDistance, (0, _reactNativeReanimated.sub)(props.horizontal ? x : y, touchInit)), (0, _reactNativeReanimated.set)(touchAbsolute, props.horizontal ? x : y)])]), (0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.neq)(panGestureState, state), [(0, _reactNativeReanimated.set)(panGestureState, state), (0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.or)((0, _reactNativeReanimated.eq)(state, _reactNativeGestureHandler.State.END), (0, _reactNativeReanimated.eq)(state, _reactNativeGestureHandler.State.CANCELLED), (0, _reactNativeReanimated.eq)(state, _reactNativeGestureHandler.State.FAILED)), onGestureRelease)])])
  }]), [activationDistance, props.horizontal, panGestureState, disabled, onGestureRelease, touchAbsolute, touchInit]);
  const onPanGestureEvent = (0, _react.useMemo)(() => (0, _reactNativeReanimated.event)([{
    nativeEvent: ({
      x,
      y
    }) => (0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.and)((0, _reactNativeReanimated.greaterThan)(activeIndexAnim, -1), (0, _reactNativeReanimated.eq)(panGestureState, _reactNativeGestureHandler.State.ACTIVE), (0, _reactNativeReanimated.not)(disabled)), [(0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.not)(hasMoved), (0, _reactNativeReanimated.set)(hasMoved, 1)), (0, _reactNativeReanimated.set)(touchAbsolute, props.horizontal ? x : y)])
  }]), [activeIndexAnim, disabled, hasMoved, panGestureState, props.horizontal, touchAbsolute]);
  const scrollHandler = (0, _react.useMemo)(() => {
    // Web doesn't seem to like animated events
    const webOnScroll = ({
      nativeEvent: {
        contentOffset: {
          x,
          y
        }
      }
    }) => {
      scrollOffset.setValue(props.horizontal ? x : y);
    };

    const mobileOnScroll = (0, _reactNativeReanimated.event)([{
      nativeEvent: ({
        contentOffset
      }) => (0, _reactNativeReanimated.block)([(0, _reactNativeReanimated.set)(scrollOffset, props.horizontal ? contentOffset.x : contentOffset.y), autoScrollNode])
    }]);
    return _constants.isWeb ? webOnScroll : mobileOnScroll;
  }, [autoScrollNode, props.horizontal, scrollOffset]);
  return /*#__PURE__*/_react.default.createElement(_DraggableFlatListContext.default, {
    activeKey: activeKey,
    onDragEnd: onDragEnd,
    keyExtractor: keyExtractor,
    horizontal: !!props.horizontal
  }, /*#__PURE__*/_react.default.createElement(_reactNativeGestureHandler.PanGestureHandler, _extends({
    ref: panGestureHandlerRef,
    hitSlop: dragHitSlop,
    onHandlerStateChange: onPanStateChange,
    onGestureEvent: onPanGestureEvent,
    simultaneousHandlers: props.simultaneousHandlers
  }, dynamicProps), /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    style: props.containerStyle,
    ref: containerRef,
    onLayout: onContainerLayout,
    onTouchEnd: onContainerTouchEnd,
    onStartShouldSetResponderCapture: onContainerTouchStart //@ts-ignore
    ,
    onClick: onContainerTouchEnd
  }, /*#__PURE__*/_react.default.createElement(_ScrollOffsetListener.default, {
    scrollOffset: scrollOffset,
    onScrollOffsetChange: ([offset]) => {
      var _props$onScrollOffset;

      scrollOffsetRef.current = offset;
      (_props$onScrollOffset = props.onScrollOffsetChange) === null || _props$onScrollOffset === void 0 ? void 0 : _props$onScrollOffset.call(props, offset);
    }
  }), /*#__PURE__*/_react.default.createElement(_PlaceholderItem.default, {
    renderPlaceholder: props.renderPlaceholder
  }), /*#__PURE__*/_react.default.createElement(AnimatedFlatList, _extends({}, props, {
    CellRendererComponent: _CellRendererComponent.default,
    ref: flatlistRef,
    onContentSizeChange: onListContentSizeChange,
    scrollEnabled: !activeKey && scrollEnabled,
    renderItem: renderItem,
    extraData: extraData,
    keyExtractor: keyExtractor,
    onScroll: scrollHandler,
    scrollEventThrottle: 1,
    simultaneousHandlers: props.simultaneousHandlers,
    removeClippedSubviews: false
  })), /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.Code, {
    dependencies: []
  }, () => (0, _reactNativeReanimated.block)([(0, _reactNativeReanimated.onChange)(isTouchActiveRef.current.native, (0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.not)(isTouchActiveRef.current.native), onGestureRelease))])))));
}

function DraggableFlatList(props) {
  return /*#__PURE__*/_react.default.createElement(_PropsContext.default, props, /*#__PURE__*/_react.default.createElement(_AnimatedValueContext.default, null, /*#__PURE__*/_react.default.createElement(_RefContext.default, null, /*#__PURE__*/_react.default.createElement(DraggableFlatListInner, props))));
}
//# sourceMappingURL=DraggableFlatList.js.map