"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = AnimatedValueProvider;
exports.useAnimatedValues = useAnimatedValues;

var _react = _interopRequireWildcard(require("react"));

var _reactNativeReanimated = require("react-native-reanimated");

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _useNode = require("../hooks/useNode");

var _PropsContext = require("./PropsContext");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

if (!_reactNativeReanimated.useValue) {
  throw new Error("Incompatible Reanimated version (useValue not found)");
}

const AnimatedValueContext = /*#__PURE__*/_react.default.createContext(undefined);

function AnimatedValueProvider({
  children
}) {
  const value = useSetupAnimatedValues();
  return /*#__PURE__*/_react.default.createElement(AnimatedValueContext.Provider, {
    value: value
  }, children);
}

function useAnimatedValues() {
  const value = (0, _react.useContext)(AnimatedValueContext);

  if (!value) {
    throw new Error("useAnimatedValues must be called from within AnimatedValueProvider!");
  }

  return value;
}

function useSetupAnimatedValues() {
  const props = (0, _PropsContext.useProps)();
  const containerSize = (0, _reactNativeReanimated.useValue)(0);
  const touchInit = (0, _reactNativeReanimated.useValue)(0); // Position of initial touch

  const activationDistance = (0, _reactNativeReanimated.useValue)(0); // Distance finger travels from initial touch to when dragging begins

  const touchAbsolute = (0, _reactNativeReanimated.useValue)(0); // Finger position on screen, relative to container

  const panGestureState = (0, _reactNativeReanimated.useValue)(_reactNativeGestureHandler.State.UNDETERMINED);
  const isTouchActiveNative = (0, _reactNativeReanimated.useValue)(0);
  const hasMoved = (0, _reactNativeReanimated.useValue)(0);
  const disabled = (0, _reactNativeReanimated.useValue)(0);
  const horizontalAnim = (0, _reactNativeReanimated.useValue)(props.horizontal ? 1 : 0);
  const activeIndexAnim = (0, _reactNativeReanimated.useValue)(-1); // Index of hovering cell

  const spacerIndexAnim = (0, _reactNativeReanimated.useValue)(-1); // Index of hovered-over cell

  const activeCellSize = (0, _reactNativeReanimated.useValue)(0); // Height or width of acctive cell

  const activeCellOffset = (0, _reactNativeReanimated.useValue)(0); // Distance between active cell and edge of container

  const isDraggingCell = (0, _useNode.useNode)((0, _reactNativeReanimated.and)(isTouchActiveNative, (0, _reactNativeReanimated.greaterThan)(activeIndexAnim, -1)));
  const scrollOffset = (0, _reactNativeReanimated.useValue)(0);
  const scrollViewSize = (0, _reactNativeReanimated.useValue)(0);
  const touchCellOffset = (0, _useNode.useNode)((0, _reactNativeReanimated.sub)(touchInit, activeCellOffset));
  const hoverAnimUnconstrained = (0, _useNode.useNode)((0, _reactNativeReanimated.sub)((0, _reactNativeReanimated.sub)(touchAbsolute, activationDistance), touchCellOffset));
  const hoverAnimConstrained = (0, _useNode.useNode)((0, _reactNativeReanimated.min)((0, _reactNativeReanimated.sub)(containerSize, activeCellSize), (0, _reactNativeReanimated.max)(0, hoverAnimUnconstrained)));
  const hoverAnim = props.dragItemOverflow ? hoverAnimUnconstrained : hoverAnimConstrained;
  const hoverOffset = (0, _useNode.useNode)((0, _reactNativeReanimated.add)(hoverAnim, scrollOffset));
  const placeholderOffset = (0, _reactNativeReanimated.useValue)(0);
  const placeholderScreenOffset = (0, _useNode.useNode)((0, _reactNativeReanimated.sub)(placeholderOffset, scrollOffset)); // Note: this could use a refactor as it combines touch state + cell animation

  const resetTouchedCell = (0, _useNode.useNode)((0, _reactNativeReanimated.block)([(0, _reactNativeReanimated.set)(touchAbsolute, 0), (0, _reactNativeReanimated.set)(touchInit, 0), (0, _reactNativeReanimated.set)(activeCellOffset, 0), (0, _reactNativeReanimated.set)(activationDistance, 0), (0, _reactNativeReanimated.set)(hasMoved, 0)]));
  const value = (0, _react.useMemo)(() => ({
    activationDistance,
    activeCellOffset,
    activeCellSize,
    activeIndexAnim,
    containerSize,
    disabled,
    hasMoved,
    horizontalAnim,
    hoverAnim,
    hoverAnimConstrained,
    hoverAnimUnconstrained,
    hoverOffset,
    isDraggingCell,
    isTouchActiveNative,
    panGestureState,
    placeholderOffset,
    placeholderScreenOffset,
    resetTouchedCell,
    scrollOffset,
    scrollViewSize,
    spacerIndexAnim,
    touchAbsolute,
    touchCellOffset,
    touchInit
  }), [activationDistance, activeCellOffset, activeCellSize, activeIndexAnim, containerSize, disabled, hasMoved, horizontalAnim, hoverAnim, hoverAnimConstrained, hoverAnimUnconstrained, hoverOffset, isDraggingCell, isTouchActiveNative, panGestureState, placeholderOffset, placeholderScreenOffset, resetTouchedCell, scrollOffset, scrollViewSize, spacerIndexAnim, touchAbsolute, touchCellOffset, touchInit]);
  return value;
}
//# sourceMappingURL=animatedValueContext.js.map