"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = RefProvider;
exports.useRefs = useRefs;

var _react = _interopRequireWildcard(require("react"));

var _constants = require("../constants");

var _PropsContext = require("./PropsContext");

var _AnimatedValueContext = require("./AnimatedValueContext");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const RefContext = /*#__PURE__*/_react.default.createContext(undefined);

function RefProvider({
  children
}) {
  const value = useSetupRefs();
  return /*#__PURE__*/_react.default.createElement(RefContext.Provider, {
    value: value
  }, children);
}

function useRefs() {
  const value = (0, _react.useContext)(RefContext);

  if (!value) {
    throw new Error("useRefs must be called from within a RefContext.Provider!");
  }

  return value;
}

function useSetupRefs() {
  const props = (0, _PropsContext.useProps)();
  const {
    onRef,
    animationConfig = _constants.DEFAULT_PROPS.animationConfig
  } = props;
  const {
    isTouchActiveNative
  } = (0, _AnimatedValueContext.useAnimatedValues)();
  const propsRef = (0, _react.useRef)(props);
  propsRef.current = props;
  const animConfig = { ..._constants.DEFAULT_PROPS.animationConfig,
    ...animationConfig
  };
  const animationConfigRef = (0, _react.useRef)(animConfig);
  animationConfigRef.current = animConfig;
  const cellDataRef = (0, _react.useRef)(new Map());
  const keyToIndexRef = (0, _react.useRef)(new Map());
  const containerRef = (0, _react.useRef)(null);
  const flatlistRef = (0, _react.useRef)(null);
  const panGestureHandlerRef = (0, _react.useRef)(null);
  const scrollOffsetRef = (0, _react.useRef)(0);
  const isTouchActiveRef = (0, _react.useRef)({
    native: isTouchActiveNative,
    js: false
  });
  (0, _react.useEffect)(() => {
    if (flatlistRef.current) onRef === null || onRef === void 0 ? void 0 : onRef(flatlistRef.current);
  }, [onRef]);
  const refs = (0, _react.useMemo)(() => ({
    animationConfigRef,
    cellDataRef,
    containerRef,
    flatlistRef,
    isTouchActiveRef,
    keyToIndexRef,
    panGestureHandlerRef,
    propsRef,
    scrollOffsetRef
  }), []);
  return refs;
}
//# sourceMappingURL=refContext.js.map