"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useAutoScroll = useAutoScroll;

var _react = require("react");

var _reactNativeReanimated = require("react-native-reanimated");

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _constants = require("../constants");

var _useNode = require("../hooks/useNode");

var _PropsContext = require("../context/PropsContext");

var _AnimatedValueContext = require("../context/AnimatedValueContext");

var _RefContext = require("../context/RefContext");

function useAutoScroll() {
  const {
    flatlistRef
  } = (0, _RefContext.useRefs)();
  const {
    autoscrollThreshold = _constants.DEFAULT_PROPS.autoscrollThreshold,
    autoscrollSpeed = _constants.DEFAULT_PROPS.autoscrollSpeed
  } = (0, _PropsContext.useProps)();
  const {
    scrollOffset,
    scrollViewSize,
    containerSize,
    hoverAnim,
    isDraggingCell,
    activeCellSize,
    panGestureState
  } = (0, _AnimatedValueContext.useAnimatedValues)();
  const isScrolledUp = (0, _useNode.useNode)((0, _reactNativeReanimated.lessOrEq)((0, _reactNativeReanimated.sub)(scrollOffset, _constants.SCROLL_POSITION_TOLERANCE), 0));
  const isScrolledDown = (0, _useNode.useNode)((0, _reactNativeReanimated.greaterOrEq)((0, _reactNativeReanimated.add)(scrollOffset, containerSize, _constants.SCROLL_POSITION_TOLERANCE), scrollViewSize));
  const distToTopEdge = (0, _useNode.useNode)((0, _reactNativeReanimated.max)(0, hoverAnim));
  const distToBottomEdge = (0, _useNode.useNode)((0, _reactNativeReanimated.max)(0, (0, _reactNativeReanimated.sub)(containerSize, (0, _reactNativeReanimated.add)(hoverAnim, activeCellSize))));
  const isAtTopEdge = (0, _useNode.useNode)((0, _reactNativeReanimated.lessOrEq)(distToTopEdge, autoscrollThreshold));
  const isAtBottomEdge = (0, _useNode.useNode)((0, _reactNativeReanimated.lessOrEq)(distToBottomEdge, autoscrollThreshold));
  const isAtEdge = (0, _useNode.useNode)((0, _reactNativeReanimated.or)(isAtBottomEdge, isAtTopEdge));
  const autoscrollParams = [distToTopEdge, distToBottomEdge, scrollOffset, isScrolledUp, isScrolledDown];
  const targetScrollOffset = (0, _reactNativeReanimated.useValue)(0);
  const resolveAutoscroll = (0, _react.useRef)();
  const isAutoScrollInProgressNative = (0, _reactNativeReanimated.useValue)(0);
  const isAutoScrollInProgress = (0, _react.useRef)({
    js: false,
    native: isAutoScrollInProgressNative
  });
  const isDraggingCellJS = (0, _react.useRef)(false);
  (0, _reactNativeReanimated.useCode)(() => (0, _reactNativeReanimated.block)([(0, _reactNativeReanimated.onChange)(isDraggingCell, (0, _reactNativeReanimated.call)([isDraggingCell], ([v]) => {
    isDraggingCellJS.current = !!v;
  }))]), []); // Ensure that only 1 call to autoscroll is active at a time

  const autoscrollLooping = (0, _react.useRef)(false);

  const onAutoscrollComplete = params => {
    var _resolveAutoscroll$cu;

    isAutoScrollInProgress.current.js = false;
    (_resolveAutoscroll$cu = resolveAutoscroll.current) === null || _resolveAutoscroll$cu === void 0 ? void 0 : _resolveAutoscroll$cu.call(resolveAutoscroll, params);
  };

  const scrollToAsync = offset => new Promise(resolve => {
    var _flatlistNode$scrollT;

    resolveAutoscroll.current = resolve;
    targetScrollOffset.setValue(offset);
    isAutoScrollInProgress.current.native.setValue(1);
    isAutoScrollInProgress.current.js = true;
    const flatlistNode = "scrollToOffset" in flatlistRef ? flatlistRef : "getNode" in flatlistRef ? //@ts-ignore backwards compat
    flatlistRef.getNode() : null;
    flatlistNode === null || flatlistNode === void 0 ? void 0 : (_flatlistNode$scrollT = flatlistNode.scrollToOffset) === null || _flatlistNode$scrollT === void 0 ? void 0 : _flatlistNode$scrollT.call(flatlistNode, {
      offset
    });
  });

  const getScrollTargetOffset = (distFromTop, distFromBottom, scrollOffset, isScrolledUp, isScrolledDown) => {
    if (isAutoScrollInProgress.current.js) return -1;
    const scrollUp = distFromTop < autoscrollThreshold;
    const scrollDown = distFromBottom < autoscrollThreshold;
    if (!(scrollUp || scrollDown) || scrollUp && isScrolledUp || scrollDown && isScrolledDown) return -1;
    const distFromEdge = scrollUp ? distFromTop : distFromBottom;
    const speedPct = 1 - distFromEdge / autoscrollThreshold; // Android scroll speed seems much faster than ios

    const speed = _constants.isAndroid ? autoscrollSpeed / 10 : autoscrollSpeed;
    const offset = speedPct * speed;
    const targetOffset = scrollUp ? Math.max(0, scrollOffset - offset) : scrollOffset + offset;
    return targetOffset;
  };

  const autoscroll = async params => {
    if (autoscrollLooping.current) {
      return;
    }

    autoscrollLooping.current = true;

    try {
      let shouldScroll = true;
      let curParams = params;

      while (shouldScroll) {
        const [distFromTop, distFromBottom, scrollOffset, isScrolledUp, isScrolledDown] = curParams;
        const targetOffset = getScrollTargetOffset(distFromTop, distFromBottom, scrollOffset, !!isScrolledUp, !!isScrolledDown);
        const scrollingUpAtTop = !!(isScrolledUp && targetOffset <= scrollOffset);
        const scrollingDownAtBottom = !!(isScrolledDown && targetOffset >= scrollOffset);
        shouldScroll = targetOffset >= 0 && isDraggingCellJS.current && !scrollingUpAtTop && !scrollingDownAtBottom;

        if (shouldScroll) {
          try {
            curParams = await scrollToAsync(targetOffset);
          } catch (err) {}
        }
      }
    } finally {
      autoscrollLooping.current = false;
    }
  };

  const checkAutoscroll = (0, _useNode.useNode)((0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.and)(isAtEdge, (0, _reactNativeReanimated.not)((0, _reactNativeReanimated.and)(isAtTopEdge, isScrolledUp)), (0, _reactNativeReanimated.not)((0, _reactNativeReanimated.and)(isAtBottomEdge, isScrolledDown)), (0, _reactNativeReanimated.eq)(panGestureState, _reactNativeGestureHandler.State.ACTIVE), (0, _reactNativeReanimated.not)(isAutoScrollInProgress.current.native)), (0, _reactNativeReanimated.call)(autoscrollParams, autoscroll)));
  (0, _reactNativeReanimated.useCode)(() => checkAutoscroll, []);
  const onScrollNode = (0, _useNode.useNode)((0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.and)(isAutoScrollInProgress.current.native, (0, _reactNativeReanimated.or)( // We've scrolled to where we want to be
  (0, _reactNativeReanimated.lessOrEq)((0, _reactNativeReanimated.abs)((0, _reactNativeReanimated.sub)(targetScrollOffset, scrollOffset)), _constants.SCROLL_POSITION_TOLERANCE), // We're at the start, but still want to scroll farther up
  (0, _reactNativeReanimated.and)(isScrolledUp, (0, _reactNativeReanimated.lessOrEq)(targetScrollOffset, scrollOffset)), // We're at the end, but still want to scroll further down
  (0, _reactNativeReanimated.and)(isScrolledDown, (0, _reactNativeReanimated.greaterOrEq)(targetScrollOffset, scrollOffset)))), [// Finish scrolling
  (0, _reactNativeReanimated.set)(isAutoScrollInProgress.current.native, 0), (0, _reactNativeReanimated.call)(autoscrollParams, onAutoscrollComplete)]));
  return onScrollNode;
}
//# sourceMappingURL=useAutoScroll.js.map