"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCellTranslate = useCellTranslate;

var _reactNativeReanimated = require("react-native-reanimated");

var _AnimatedValueContext = require("../context/AnimatedValueContext");

var _RefContext = require("../context/RefContext");

var _procs = require("../procs");

var _useSpring = require("./useSpring");

var _useNode = require("../hooks/useNode");

var _DraggableFlatListContext = require("../context/DraggableFlatListContext");

function useCellTranslate({
  cellIndex,
  cellSize,
  cellOffset
}) {
  const {
    activeIndexAnim,
    activeCellSize,
    hoverAnim,
    scrollOffset,
    spacerIndexAnim,
    placeholderOffset,
    isDraggingCell,
    resetTouchedCell,
    disabled
  } = (0, _AnimatedValueContext.useAnimatedValues)();
  const {
    animationConfigRef
  } = (0, _RefContext.useRefs)();
  const {
    onDragEnd
  } = (0, _DraggableFlatListContext.useDraggableFlatListContext)();
  const cellSpring = (0, _useSpring.useSpring)({
    config: animationConfigRef.current
  });
  const {
    clock,
    state,
    config
  } = cellSpring;
  const isAfterActive = (0, _reactNativeReanimated.useValue)(0);
  const isClockRunning = (0, _useNode.useNode)((0, _reactNativeReanimated.clockRunning)(clock));
  const runSpring = (0, _useNode.useNode)((0, _procs.springFill)(clock, state, config)); // Even though this is the same value as hoverOffset passed via context
  // the android context value lags behind the actual value on autoscroll

  const cellHoverOffset = (0, _useNode.useNode)((0, _reactNativeReanimated.add)(hoverAnim, scrollOffset));
  const onFinished = (0, _useNode.useNode)((0, _reactNativeReanimated.cond)(isClockRunning, [(0, _reactNativeReanimated.stopClock)(clock), (0, _reactNativeReanimated.cond)((0, _reactNativeReanimated.eq)(cellIndex, activeIndexAnim), [resetTouchedCell, (0, _reactNativeReanimated.call)([activeIndexAnim, spacerIndexAnim], onDragEnd)])]));
  const prevTrans = (0, _reactNativeReanimated.useValue)(0);
  const prevSpacerIndex = (0, _reactNativeReanimated.useValue)(-1);
  const prevIsDraggingCell = (0, _reactNativeReanimated.useValue)(0);
  const cellTranslate = (0, _useNode.useNode)((0, _procs.setupCell)(cellIndex, cellSize, cellOffset, isAfterActive, prevTrans, prevSpacerIndex, activeIndexAnim, activeCellSize, cellHoverOffset, spacerIndexAnim, //@ts-ignore
  config.toValue, state.position, state.time, state.finished, runSpring, onFinished, isDraggingCell, placeholderOffset, prevIsDraggingCell, clock, disabled)); // This is required to continually evaluate values

  (0, _reactNativeReanimated.useCode)(() => (0, _reactNativeReanimated.block)([(0, _reactNativeReanimated.onChange)(cellTranslate, []), (0, _reactNativeReanimated.onChange)(prevTrans, []), (0, _reactNativeReanimated.onChange)(cellSize, []), (0, _reactNativeReanimated.onChange)(cellOffset, [])]), []);
  return state.position;
}
//# sourceMappingURL=useCellTranslate.js.map