function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React, { useCallback, useEffect, useLayoutEffect, useMemo, useRef } from "react";
import { findNodeHandle } from "react-native";
import Animated, { cond, useValue } from "react-native-reanimated";
import { useDraggableFlatListContext } from "../context/DraggableFlatListContext";
import { isAndroid, isIOS, isReanimatedV2, isWeb } from "../constants";
import { useCellTranslate } from "../hooks/useCellTranslate";
import { typedMemo } from "../utils";
import { useRefs } from "../context/RefContext";
import { useAnimatedValues } from "../context/AnimatedValueContext";
import CellProvider from "../context/CellContext";

function CellRendererComponent(props) {
  const {
    item,
    index,
    onLayout,
    children
  } = props;
  const currentIndexAnim = useValue(index);
  useLayoutEffect(() => {
    currentIndexAnim.setValue(index);
  }, [index, currentIndexAnim]);
  const viewRef = useRef(null);
  const {
    cellDataRef,
    propsRef,
    scrollOffsetRef,
    containerRef
  } = useRefs();
  const {
    horizontalAnim
  } = useAnimatedValues();
  const {
    activeKey,
    keyExtractor,
    horizontal
  } = useDraggableFlatListContext();
  const key = keyExtractor(item, index);
  const offset = useValue(-1);
  const size = useValue(-1);
  const translate = useCellTranslate({
    cellOffset: offset,
    cellSize: size,
    cellIndex: currentIndexAnim
  });
  useMemo(() => {
    // prevent flicker on web
    if (isWeb) translate.setValue(0);
  }, [index]); //eslint-disable-line react-hooks/exhaustive-deps

  const isActive = activeKey === key;
  const style = useMemo(() => ({
    transform: [{
      translateX: cond(horizontalAnim, translate, 0)
    }, {
      translateY: cond(horizontalAnim, 0, translate)
    }]
  }), [horizontalAnim, translate]);
  const updateCellMeasurements = useCallback(() => {
    var _viewRef$current;

    const onSuccess = (x, y, w, h) => {
      if (isWeb && horizontal) x += scrollOffsetRef.current;
      const cellOffset = horizontal ? x : y;
      const cellSize = horizontal ? w : h;
      cellDataRef.current.set(key, {
        measurements: {
          size: cellSize,
          offset: cellOffset
        }
      });
      size.setValue(cellSize);
      offset.setValue(cellOffset);
    };

    const onFail = () => {
      var _propsRef$current;

      if ((_propsRef$current = propsRef.current) !== null && _propsRef$current !== void 0 && _propsRef$current.debug) {
        console.log(`## on measure fail, index: ${index}`);
      }
    }; // findNodeHandle is being deprecated. This is no longer necessary if using reanimated v2
    // remove once v1 is no longer supported


    const containerNode = containerRef.current;
    const viewNode = isReanimatedV2 ? viewRef.current : (_viewRef$current = viewRef.current) === null || _viewRef$current === void 0 ? void 0 : _viewRef$current.getNode(); //@ts-ignore

    const nodeHandle = isReanimatedV2 ? containerNode : findNodeHandle(containerNode);

    if (viewNode && nodeHandle) {
      //@ts-ignore
      viewNode.measureLayout(nodeHandle, onSuccess, onFail);
    }
  }, [cellDataRef, horizontal, index, key, offset, propsRef, size, scrollOffsetRef, containerRef]);
  useEffect(() => {
    if (isWeb) {
      // onLayout isn't called on web when the cell index changes, so we manually re-measure
      updateCellMeasurements();
    }
  }, [index, updateCellMeasurements]);
  const onCellLayout = useCallback(e => {
    updateCellMeasurements();
    onLayout(e);
  }, [updateCellMeasurements, onLayout]); // changing zIndex crashes android:
  // https://github.com/facebook/react-native/issues/28751

  return /*#__PURE__*/React.createElement(Animated.View, _extends({}, props, {
    ref: viewRef,
    onLayout: onCellLayout,
    style: [isAndroid && {
      elevation: isActive ? 1 : 0
    }, {
      flexDirection: horizontal ? "row" : "column"
    }, (isWeb || isIOS) && {
      zIndex: isActive ? 999 : 0
    }],
    pointerEvents: activeKey ? "none" : "auto"
  }), /*#__PURE__*/React.createElement(Animated.View, {
    // Including both animated styles and non-animated styles causes react-native-web
    // to ignore updates in non-animated styles. Solution is to separate anima
    style: style
  }, /*#__PURE__*/React.createElement(CellProvider, {
    isActive: isActive
  }, children)));
}

export default typedMemo(CellRendererComponent);
//# sourceMappingURL=CellRendererComponent.js.map