function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React, { useCallback, useLayoutEffect, useMemo, useState } from "react";
import { PanGestureHandler, State as GestureState, FlatList } from "react-native-gesture-handler";
import Animated, { and, block, call, cond, eq, event, greaterThan, neq, not, onChange, or, set, sub } from "react-native-reanimated";
import CellRendererComponent from "./CellRendererComponent";
import { DEFAULT_PROPS, isReanimatedV2, isWeb } from "../constants";
import PlaceholderItem from "./PlaceholderItem";
import RowItem from "./RowItem";
import ScrollOffsetListener from "./ScrollOffsetListener";
import { useAutoScroll } from "../hooks/useAutoScroll";
import { useNode } from "../hooks/useNode";
import PropsProvider from "../context/PropsContext";
import AnimatedValueProvider, { useAnimatedValues } from "../context/AnimatedValueContext";
import RefProvider, { useRefs } from "../context/RefContext";
import DraggableFlatListProvider from "../context/DraggableFlatListContext";
const AnimatedFlatList = Animated.createAnimatedComponent(FlatList);

function DraggableFlatListInner(props) {
  const {
    cellDataRef,
    containerRef,
    flatlistRef,
    isTouchActiveRef,
    keyToIndexRef,
    panGestureHandlerRef,
    propsRef,
    scrollOffsetRef
  } = useRefs();
  const {
    activationDistance,
    activeCellOffset,
    activeCellSize,
    activeIndexAnim,
    containerSize,
    disabled,
    hasMoved,
    panGestureState,
    resetTouchedCell,
    scrollOffset,
    scrollViewSize,
    spacerIndexAnim,
    touchAbsolute,
    touchInit
  } = useAnimatedValues();
  const {
    dragHitSlop = DEFAULT_PROPS.dragHitSlop,
    scrollEnabled = DEFAULT_PROPS.scrollEnabled,
    activationDistance: activationDistanceProp = DEFAULT_PROPS.activationDistance
  } = props;
  const [activeKey, setActiveKey] = useState(null);
  const keyExtractor = useCallback((item, index) => {
    if (propsRef.current.keyExtractor) return propsRef.current.keyExtractor(item, index);else throw new Error("You must provide a keyExtractor to DraggableFlatList");
  }, [propsRef]);
  useLayoutEffect(() => {
    props.data.forEach((d, i) => {
      const key = keyExtractor(d, i);
      keyToIndexRef.current.set(key, i);
    });
  }, [props.data, keyExtractor, keyToIndexRef]);
  const drag = useCallback(activeKey => {
    if (!isTouchActiveRef.current.js) return;
    const index = keyToIndexRef.current.get(activeKey);
    const cellData = cellDataRef.current.get(activeKey);

    if (cellData) {
      activeCellOffset.setValue(cellData.measurements.offset - scrollOffsetRef.current);
      activeCellSize.setValue(cellData.measurements.size);
    }

    const {
      onDragBegin
    } = propsRef.current;

    if (index !== undefined) {
      spacerIndexAnim.setValue(index);
      activeIndexAnim.setValue(index);
      setActiveKey(activeKey);
      onDragBegin === null || onDragBegin === void 0 ? void 0 : onDragBegin(index);
    }
  }, [isTouchActiveRef, keyToIndexRef, cellDataRef, propsRef, activeCellOffset, scrollOffsetRef, activeCellSize, spacerIndexAnim, activeIndexAnim]);
  const autoScrollNode = useAutoScroll();

  const onContainerLayout = () => {
    var _containerRef$current;

    const containerNode = isReanimatedV2 ? containerRef.current : (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.getNode(); //@ts-ignore

    containerNode === null || containerNode === void 0 ? void 0 : containerNode.measure((_x, _y, w, h) => {
      containerSize.setValue(props.horizontal ? w : h);
    });
  };

  const onListContentSizeChange = (w, h) => {
    var _props$onContentSizeC;

    scrollViewSize.setValue(props.horizontal ? w : h);
    (_props$onContentSizeC = props.onContentSizeChange) === null || _props$onContentSizeC === void 0 ? void 0 : _props$onContentSizeC.call(props, w, h);
  };

  const onContainerTouchStart = () => {
    isTouchActiveRef.current.js = true;
    isTouchActiveRef.current.native.setValue(1);
    return false;
  };

  const onContainerTouchEnd = () => {
    isTouchActiveRef.current.js = false;
    isTouchActiveRef.current.native.setValue(0);
  };

  let dynamicProps = {};

  if (activationDistanceProp) {
    const activeOffset = [-activationDistanceProp, activationDistanceProp];
    dynamicProps = props.horizontal ? {
      activeOffsetX: activeOffset
    } : {
      activeOffsetY: activeOffset
    };
  }

  const extraData = useMemo(() => ({
    activeKey,
    extraData: props.extraData
  }), [activeKey, props.extraData]);
  const renderItem = useCallback(({
    item,
    index
  }) => {
    return /*#__PURE__*/React.createElement(RowItem, {
      item: item,
      itemKey: keyExtractor(item, index),
      renderItem: props.renderItem,
      drag: drag,
      extraData: props.extraData
    });
  }, [props.renderItem, props.extraData, drag, keyExtractor]);
  const resetHoverState = useCallback(() => {
    activeIndexAnim.setValue(-1);
    spacerIndexAnim.setValue(-1);
    touchAbsolute.setValue(0);
    disabled.setValue(0);
    requestAnimationFrame(() => {
      setActiveKey(null);
    });
  }, [activeIndexAnim, spacerIndexAnim, touchAbsolute, disabled]);

  const onRelease = ([index]) => {
    var _props$onRelease;

    // This shouldn't be necessary but seems to fix a bug where sometimes
    // native values wouldn't update
    isTouchActiveRef.current.native.setValue(0);
    (_props$onRelease = props.onRelease) === null || _props$onRelease === void 0 ? void 0 : _props$onRelease.call(props, index);
  };

  const onDragEnd = useCallback(([from, to]) => {
    const {
      onDragEnd,
      data
    } = propsRef.current;

    if (onDragEnd) {
      const newData = [...data];

      if (from !== to) {
        newData.splice(from, 1);
        newData.splice(to, 0, data[from]);
      }

      onDragEnd({
        from,
        to,
        data: newData
      });
    }

    resetHoverState();
  }, [resetHoverState, propsRef]);
  const onGestureRelease = useNode(cond(greaterThan(activeIndexAnim, -1), [set(disabled, 1), set(isTouchActiveRef.current.native, 0), call([activeIndexAnim], onRelease), cond(not(hasMoved), call([activeIndexAnim], resetHoverState))], [call([activeIndexAnim], resetHoverState), resetTouchedCell]));
  const onPanStateChange = useMemo(() => event([{
    nativeEvent: ({
      state,
      x,
      y
    }) => block([cond(and(neq(state, panGestureState), not(disabled)), [cond(or(eq(state, GestureState.BEGAN), // Called on press in on Android, NOT on ios!
    // GestureState.BEGAN may be skipped on fast swipes
    and(eq(state, GestureState.ACTIVE), neq(panGestureState, GestureState.BEGAN))), [set(touchAbsolute, props.horizontal ? x : y), set(touchInit, touchAbsolute)]), cond(eq(state, GestureState.ACTIVE), [set(activationDistance, sub(props.horizontal ? x : y, touchInit)), set(touchAbsolute, props.horizontal ? x : y)])]), cond(neq(panGestureState, state), [set(panGestureState, state), cond(or(eq(state, GestureState.END), eq(state, GestureState.CANCELLED), eq(state, GestureState.FAILED)), onGestureRelease)])])
  }]), [activationDistance, props.horizontal, panGestureState, disabled, onGestureRelease, touchAbsolute, touchInit]);
  const onPanGestureEvent = useMemo(() => event([{
    nativeEvent: ({
      x,
      y
    }) => cond(and(greaterThan(activeIndexAnim, -1), eq(panGestureState, GestureState.ACTIVE), not(disabled)), [cond(not(hasMoved), set(hasMoved, 1)), set(touchAbsolute, props.horizontal ? x : y)])
  }]), [activeIndexAnim, disabled, hasMoved, panGestureState, props.horizontal, touchAbsolute]);
  const scrollHandler = useMemo(() => {
    // Web doesn't seem to like animated events
    const webOnScroll = ({
      nativeEvent: {
        contentOffset: {
          x,
          y
        }
      }
    }) => {
      scrollOffset.setValue(props.horizontal ? x : y);
    };

    const mobileOnScroll = event([{
      nativeEvent: ({
        contentOffset
      }) => block([set(scrollOffset, props.horizontal ? contentOffset.x : contentOffset.y), autoScrollNode])
    }]);
    return isWeb ? webOnScroll : mobileOnScroll;
  }, [autoScrollNode, props.horizontal, scrollOffset]);
  return /*#__PURE__*/React.createElement(DraggableFlatListProvider, {
    activeKey: activeKey,
    onDragEnd: onDragEnd,
    keyExtractor: keyExtractor,
    horizontal: !!props.horizontal
  }, /*#__PURE__*/React.createElement(PanGestureHandler, _extends({
    ref: panGestureHandlerRef,
    hitSlop: dragHitSlop,
    onHandlerStateChange: onPanStateChange,
    onGestureEvent: onPanGestureEvent,
    simultaneousHandlers: props.simultaneousHandlers
  }, dynamicProps), /*#__PURE__*/React.createElement(Animated.View, {
    style: props.containerStyle,
    ref: containerRef,
    onLayout: onContainerLayout,
    onTouchEnd: onContainerTouchEnd,
    onStartShouldSetResponderCapture: onContainerTouchStart //@ts-ignore
    ,
    onClick: onContainerTouchEnd
  }, /*#__PURE__*/React.createElement(ScrollOffsetListener, {
    scrollOffset: scrollOffset,
    onScrollOffsetChange: ([offset]) => {
      var _props$onScrollOffset;

      scrollOffsetRef.current = offset;
      (_props$onScrollOffset = props.onScrollOffsetChange) === null || _props$onScrollOffset === void 0 ? void 0 : _props$onScrollOffset.call(props, offset);
    }
  }), /*#__PURE__*/React.createElement(PlaceholderItem, {
    renderPlaceholder: props.renderPlaceholder
  }), /*#__PURE__*/React.createElement(AnimatedFlatList, _extends({}, props, {
    CellRendererComponent: CellRendererComponent,
    ref: flatlistRef,
    onContentSizeChange: onListContentSizeChange,
    scrollEnabled: !activeKey && scrollEnabled,
    renderItem: renderItem,
    extraData: extraData,
    keyExtractor: keyExtractor,
    onScroll: scrollHandler,
    scrollEventThrottle: 1,
    simultaneousHandlers: props.simultaneousHandlers,
    removeClippedSubviews: false
  })), /*#__PURE__*/React.createElement(Animated.Code, {
    dependencies: []
  }, () => block([onChange(isTouchActiveRef.current.native, cond(not(isTouchActiveRef.current.native), onGestureRelease))])))));
}

export default function DraggableFlatList(props) {
  return /*#__PURE__*/React.createElement(PropsProvider, props, /*#__PURE__*/React.createElement(AnimatedValueProvider, null, /*#__PURE__*/React.createElement(RefProvider, null, /*#__PURE__*/React.createElement(DraggableFlatListInner, props))));
}
//# sourceMappingURL=DraggableFlatList.js.map