import React, { useCallback } from "react";
import { StyleSheet } from "react-native";
import Animated, { call, set, useCode, useValue, onChange, greaterThan, cond } from "react-native-reanimated";
import { useAnimatedValues } from "../context/AnimatedValueContext";
import { useDraggableFlatListContext } from "../context/DraggableFlatListContext";
import { useProps } from "../context/PropsContext";
import { useRefs } from "../context/RefContext";
import { useNode } from "../hooks/useNode";
import { typedMemo } from "../utils";

function PlaceholderItem({
  renderPlaceholder
}) {
  var _propsRef$current2;

  const {
    activeCellSize,
    placeholderScreenOffset,
    spacerIndexAnim
  } = useAnimatedValues();
  const {
    keyToIndexRef,
    propsRef
  } = useRefs();
  const {
    activeKey
  } = useDraggableFlatListContext();
  const {
    horizontal
  } = useProps(); // for some reason using placeholderScreenOffset directly is buggy

  const translate = useValue(0);
  const onPlaceholderIndexChange = useCallback(index => {
    var _propsRef$current$onP, _propsRef$current;

    (_propsRef$current$onP = (_propsRef$current = propsRef.current).onPlaceholderIndexChange) === null || _propsRef$current$onP === void 0 ? void 0 : _propsRef$current$onP.call(_propsRef$current, index);
  }, [propsRef]);
  useCode(() => onChange(spacerIndexAnim, call([spacerIndexAnim], ([i]) => {
    onPlaceholderIndexChange(i);
  })), []);
  const translateKey = horizontal ? "translateX" : "translateY";
  const sizeKey = horizontal ? "width" : "height";
  const opacity = useNode(cond(greaterThan(spacerIndexAnim, -1), 1, 0));
  const activeIndex = activeKey ? keyToIndexRef.current.get(activeKey) : undefined;
  const activeItem = activeIndex === undefined ? null : (_propsRef$current2 = propsRef.current) === null || _propsRef$current2 === void 0 ? void 0 : _propsRef$current2.data[activeIndex];
  const animStyle = {
    opacity,
    [sizeKey]: activeCellSize,
    transform: [{
      [translateKey]: translate
    }]
  };
  return /*#__PURE__*/React.createElement(Animated.View, {
    pointerEvents: activeKey ? "auto" : "none",
    style: [StyleSheet.absoluteFill, animStyle]
  }, !activeItem || activeIndex === undefined ? null : renderPlaceholder === null || renderPlaceholder === void 0 ? void 0 : renderPlaceholder({
    item: activeItem,
    index: activeIndex
  }), /*#__PURE__*/React.createElement(Animated.Code, null, () => set(translate, placeholderScreenOffset)));
}

export default typedMemo(PlaceholderItem);
//# sourceMappingURL=PlaceholderItem.js.map