import React, { useCallback, useRef } from "react";
import { useDraggableFlatListContext } from "../context/DraggableFlatListContext";
import { useRefs } from "../context/RefContext";
import { typedMemo } from "../utils";

function RowItem(props) {
  const propsRef = useRef(props);
  propsRef.current = props;
  const {
    activeKey
  } = useDraggableFlatListContext();
  const activeKeyRef = useRef(activeKey);
  activeKeyRef.current = activeKey;
  const {
    keyToIndexRef
  } = useRefs();
  const drag = useCallback(() => {
    const {
      drag,
      itemKey,
      debug
    } = propsRef.current;

    if (activeKeyRef.current) {
      // already dragging an item, noop
      if (debug) console.log("## attempt to drag item while another item is already active, noop");
    }

    drag(itemKey);
  }, []);
  const {
    renderItem,
    item,
    itemKey
  } = props;
  return /*#__PURE__*/React.createElement(MemoizedInner, {
    isActive: activeKey === itemKey,
    drag: drag,
    renderItem: renderItem,
    item: item,
    index: keyToIndexRef.current.get(itemKey)
  });
}

export default typedMemo(RowItem);

function Inner({
  isActive,
  item,
  drag,
  index,
  renderItem
}) {
  return renderItem({
    isActive,
    item,
    drag,
    index
  });
}

const MemoizedInner = typedMemo(Inner);
//# sourceMappingURL=RowItem.js.map