import React, { useContext, useEffect } from "react";
import { useMemo, useRef } from "react";
import { DEFAULT_PROPS } from "../constants";
import { useProps } from "./PropsContext";
import { useAnimatedValues } from "./AnimatedValueContext";
const RefContext = /*#__PURE__*/React.createContext(undefined);
export default function RefProvider({
  children
}) {
  const value = useSetupRefs();
  return /*#__PURE__*/React.createElement(RefContext.Provider, {
    value: value
  }, children);
}
export function useRefs() {
  const value = useContext(RefContext);

  if (!value) {
    throw new Error("useRefs must be called from within a RefContext.Provider!");
  }

  return value;
}

function useSetupRefs() {
  const props = useProps();
  const {
    onRef,
    animationConfig = DEFAULT_PROPS.animationConfig
  } = props;
  const {
    isTouchActiveNative
  } = useAnimatedValues();
  const propsRef = useRef(props);
  propsRef.current = props;
  const animConfig = { ...DEFAULT_PROPS.animationConfig,
    ...animationConfig
  };
  const animationConfigRef = useRef(animConfig);
  animationConfigRef.current = animConfig;
  const cellDataRef = useRef(new Map());
  const keyToIndexRef = useRef(new Map());
  const containerRef = useRef(null);
  const flatlistRef = useRef(null);
  const panGestureHandlerRef = useRef(null);
  const scrollOffsetRef = useRef(0);
  const isTouchActiveRef = useRef({
    native: isTouchActiveNative,
    js: false
  });
  useEffect(() => {
    if (flatlistRef.current) onRef === null || onRef === void 0 ? void 0 : onRef(flatlistRef.current);
  }, [onRef]);
  const refs = useMemo(() => ({
    animationConfigRef,
    cellDataRef,
    containerRef,
    flatlistRef,
    isTouchActiveRef,
    keyToIndexRef,
    panGestureHandlerRef,
    propsRef,
    scrollOffsetRef
  }), []);
  return refs;
}
//# sourceMappingURL=refContext.js.map