"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAutoScroll = void 0;
var react_native_reanimated_1 = require("react-native-reanimated");
var constants_1 = require("./constants");
function useAutoScroll(_a) {
    var scrollOffset = _a.scrollOffset, scrollViewSize = _a.scrollViewSize, containerSize = _a.containerSize, hoverAnim = _a.hoverAnim, isPressedIn = _a.isPressedIn, activeCellSize = _a.activeCellSize, flatlistRef = _a.flatlistRef, _b = _a.autoscrollThreshold, autoscrollThreshold = _b === void 0 ? constants_1.DEFAULT_PROPS.autoscrollThreshold : _b, _c = _a.autoscrollSpeed, autoscrollSpeed = _c === void 0 ? constants_1.DEFAULT_PROPS.autoscrollSpeed : _c;
    var isScrolledUp = react_native_reanimated_1.useDerivedValue(function () {
        return scrollOffset.value - constants_1.SCROLL_POSITION_TOLERANCE <= 0;
    });
    var isScrolledDown = react_native_reanimated_1.useDerivedValue(function () {
        return (scrollOffset.value + containerSize.value + constants_1.SCROLL_POSITION_TOLERANCE >=
            scrollViewSize.value);
    });
    var distToTopEdge = react_native_reanimated_1.useDerivedValue(function () {
        return Math.max(0, hoverAnim.value);
    });
    var distToBottomEdge = react_native_reanimated_1.useDerivedValue(function () {
        return Math.max(0, containerSize.value - (hoverAnim.value + activeCellSize.value));
    });
    var scrollTarget = react_native_reanimated_1.useSharedValue(0);
    var isAutoscrolling = react_native_reanimated_1.useSharedValue(false);
    var nextScrollTarget = react_native_reanimated_1.useDerivedValue(function () {
        var scrollUp = isPressedIn.value && distToTopEdge.value < autoscrollThreshold;
        var scrollDown = isPressedIn.value && (distToBottomEdge.value < autoscrollThreshold);
        var attemptToScroll = scrollUp || scrollDown;
        // console.log("SCROLL UP?", scrollUp)
        // console.log("SCROLL DOWN?", scrollDown)
        // console.log("PRESSED IN???", isPressedIn.value)
        if (!isPressedIn.value ||
            !attemptToScroll ||
            (scrollUp && isScrolledUp.value) ||
            (scrollDown && isScrolledDown.value)) {
            return -1;
        }
        var distFromEdge = scrollUp
            ? distToTopEdge.value
            : distToBottomEdge.value;
        var speedPct = 1 - distFromEdge / autoscrollThreshold;
        var offset = speedPct * autoscrollSpeed;
        var targetOffset = scrollUp
            ? Math.max(0, scrollOffset.value - offset)
            : scrollOffset.value + offset;
        return targetOffset;
    });
    react_native_reanimated_1.useDerivedValue(function () {
        var hasReachedTarget = Math.abs(scrollOffset.value - scrollTarget.value) <
            constants_1.SCROLL_POSITION_TOLERANCE;
        var hasReachedEdge = isScrolledUp.value || isScrolledDown.value;
        if (hasReachedTarget || hasReachedEdge) {
            isAutoscrolling.value = false;
        }
    });
    var jsScrollTo = function (_a) {
        var _b;
        var target = _a.target;
        (_b = flatlistRef.current) === null || _b === void 0 ? void 0 : _b.scrollToOffset({ offset: target });
    };
    react_native_reanimated_1.useDerivedValue(function () {
        if (!isPressedIn.value) {
            isAutoscrolling.value = false;
            return;
        }
        if (nextScrollTarget.value !== -1 &&
            nextScrollTarget.value !== scrollTarget.value &&
            !isAutoscrolling.value) {
            isAutoscrolling.value = true;
            scrollTarget.value = nextScrollTarget.value;
            // Reanimated scrollTo has been really unstable, use custom js scrollTo for the time being
            react_native_reanimated_1.runOnJS(jsScrollTo)({ target: scrollTarget.value });
        }
    });
}
exports.useAutoScroll = useAutoScroll;
