"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCellTranslate = void 0;
var react_native_reanimated_1 = require("react-native-reanimated");
var context_1 = require("./context");
function useCellTranslate(_a) {
    var cellIndex = _a.cellIndex, cellSize = _a.cellSize, cellOffset = _a.cellOffset;
    var _b = context_1.useStaticValues(), activeIndexAnim = _b.activeIndexAnim, activeCellSize = _b.activeCellSize, hoverOffset = _b.hoverOffset, isHovering = _b.isHovering, spacerIndexAnim = _b.spacerIndexAnim, placeholderOffset = _b.placeholderOffset, animationConfigRef = _b.animationConfigRef, hoverComponentTranslate = _b.hoverComponentTranslate, scrollOffset = _b.scrollOffset;
    var isActiveVisible = context_1.useActiveKey().isActiveVisible;
    var isActiveCell = react_native_reanimated_1.useDerivedValue(function () {
        return cellIndex.value === activeIndexAnim.value;
    });
    react_native_reanimated_1.useDerivedValue(function () {
        // Determining spacer index is hard to visualize. See diagram: https://i.imgur.com/jRPf5t3.jpg
        var isAfterActive = cellIndex.value > activeIndexAnim.value;
        var isBeforeActive = cellIndex.value < activeIndexAnim.value;
        var hoverPlusActiveSize = hoverOffset.value + activeCellSize.value;
        var offsetPlusHalfSize = cellOffset.value + cellSize.value / 2;
        var offsetPlusSize = cellOffset.value + cellSize.value;
        var result = -1;
        if (isAfterActive) {
            if (hoverPlusActiveSize >= cellOffset.value &&
                hoverPlusActiveSize < offsetPlusHalfSize) {
                // bottom edge of active cell overlaps top half of current cell
                result = cellIndex.value - 1;
            }
            else if (hoverPlusActiveSize >= offsetPlusHalfSize &&
                hoverPlusActiveSize < offsetPlusSize) {
                // bottom edge of active cell overlaps bottom half of current cell
                result = cellIndex.value;
            }
        }
        else if (isBeforeActive) {
            if (hoverOffset.value < offsetPlusSize &&
                hoverOffset.value >= offsetPlusHalfSize) {
                // top edge of active cell overlaps bottom half of current cell
                result = cellIndex.value + 1;
            }
            else if (hoverOffset.value >= cellOffset.value &&
                hoverOffset.value < offsetPlusHalfSize) {
                // top edge of active cell overlaps top half of current cell
                result = cellIndex.value;
            }
        }
        if (result !== -1 && isHovering.value && result !== spacerIndexAnim.value) {
            spacerIndexAnim.value = result;
        }
        if (!isHovering.value && spacerIndexAnim.value !== -1) {
            spacerIndexAnim.value = -1;
        }
        return spacerIndexAnim.value;
    }, []);
    react_native_reanimated_1.useAnimatedReaction(function () {
        var isActiveSpacerIndex = isHovering.value &&
            cellSize.value !== -1 &&
            cellOffset.value !== -1 &&
            spacerIndexAnim.value === cellIndex.value;
        return isActiveSpacerIndex;
    }, function (result, prev) {
        if (result && result !== prev) {
            var isAfterActive = cellIndex.value > activeIndexAnim.value;
            var newPlaceholderOffset = isAfterActive
                ? cellSize.value + (cellOffset.value - activeCellSize.value)
                : cellOffset.value;
            placeholderOffset.value = newPlaceholderOffset;
        }
    });
    var translate = react_native_reanimated_1.useDerivedValue(function () {
        // Active cell follows touch
        if (isActiveCell.value)
            return (hoverComponentTranslate.value - cellOffset.value + scrollOffset.value);
        // Translate cell down if it is before active index and active cell has passed it.
        // Translate cell up if it is after the active index and active cell has passed it.
        var isAfterActive = cellIndex.value > activeIndexAnim.value;
        var shouldTranslate = isAfterActive
            ? cellIndex.value <= spacerIndexAnim.value
            : cellIndex.value >= spacerIndexAnim.value;
        if (shouldTranslate) {
            return activeCellSize.value * (isAfterActive ? -1 : 1);
        }
        else {
            return 0;
        }
    });
    var lastKnownTranslate = react_native_reanimated_1.useSharedValue(0);
    react_native_reanimated_1.useDerivedValue(function () {
        if (!isActiveVisible)
            lastKnownTranslate.value = 0;
    });
    var springTranslate = react_native_reanimated_1.useDerivedValue(function () {
        if (isHovering.value)
            lastKnownTranslate.value = translate.value;
        if (isActiveCell.value)
            return translate.value;
        return isHovering.value
            ? react_native_reanimated_1.withSpring(translate.value, animationConfigRef.current)
            : isActiveVisible
                ? lastKnownTranslate.value
                : 0;
    });
    return springTranslate;
}
exports.useCellTranslate = useCellTranslate;
